@@ -41,7 +41,7 @@
 #include "gdm-settings-direct.h"
 #include "gdm-settings-keys.h"
 
-#include "gdm-slave-proxy.h"
+#include "gdm-slave-job.h"
 #include "gdm-slave-glue.h"
 #include "gdm-dbus-util.h"
 
@@ -71,9 +71,9 @@ struct GdmDisplayPrivate
         gboolean              is_local;
         guint                 finish_idle_id;
 
-        GdmSlaveProxy        *slave_proxy;
+        GdmSlaveJob          *slave_job;
         char                 *slave_bus_name;
-        GdmDBusSlave         *slave_bus_proxy;
+        GdmDBusSlave         *slave_proxy;
         int                   slave_name_id;
         GDBusConnection      *connection;
         GdmDisplayAccessFile *user_access_file;
@@ -134,10 +134,10 @@ gdm_display_get_status (GdmDisplay *display)
         return display->priv->status;
 }
 
-char *
+const char *
 gdm_display_get_session_id (GdmDisplay *display)
 {
-        return g_strdup (display->priv->session_id);
+        return display->priv->session_id;
 }
 
 static GdmDisplayAccessFile *
@@ -225,7 +225,14 @@ gdm_display_real_add_user_authorization (GdmDisplay *display,
         gboolean              res;
 
         g_return_val_if_fail (GDM_IS_DISPLAY (display), FALSE);
-        g_return_val_if_fail (display->priv->access_file != NULL, FALSE);
+
+        if (display->priv->user_access_file != NULL) {
+                g_set_error (error,
+                             G_DBUS_ERROR,
+                             G_DBUS_ERROR_ACCESS_DENIED,
+                             "user access already assigned");
+                return FALSE;
+        }
 
         g_debug ("GdmDisplay: Adding user authorization for %s", username);
 
@@ -294,14 +301,14 @@ gdm_display_real_set_slave_bus_name_finish (GObject      *source_object,
                                             GAsyncResult *res,
                                             gpointer      user_data)
 {
-    GdmDisplay *display = user_data;
+        GdmDisplay *display = user_data;
 
-    display->priv->slave_bus_proxy = GDM_DBUS_SLAVE (gdm_dbus_slave_proxy_new_finish (res, NULL));
-    g_object_bind_property (G_OBJECT (display->priv->slave_bus_proxy),
-                            "session-id",
-                            G_OBJECT (display),
-                            "session-id",
-                            G_BINDING_DEFAULT);
+        display->priv->slave_proxy = GDM_DBUS_SLAVE (gdm_dbus_slave_proxy_new_finish (res, NULL));
+        g_object_bind_property (G_OBJECT (display->priv->slave_proxy),
+                                "session-id",
+                                G_OBJECT (display),
+                                "session-id",
+                                G_BINDING_DEFAULT);
 }
 
 static gboolean
@@ -324,7 +331,7 @@ gdm_display_real_set_slave_bus_name (GdmDisplay *display,
                                                                        g_object_ref (display),
                                                                        NULL);
 
-        g_clear_object (&display->priv->slave_bus_proxy);
+        g_clear_object (&display->priv->slave_proxy);
         gdm_dbus_slave_proxy_new (display->priv->connection,
                                   G_DBUS_PROXY_FLAGS_NONE,
                                   name,
@@ -431,20 +438,38 @@ gdm_display_real_get_timed_login_details (GdmDisplay *display,
 
 gboolean
 gdm_display_get_timed_login_details (GdmDisplay *display,
-                                     gboolean   *enabled,
-                                     char      **username,
-                                     int        *delay,
+                                     gboolean   *out_enabled,
+                                     char      **out_username,
+                                     int        *out_delay,
                                      GError    **error)
 {
+        gboolean enabled;
+        char *username;
+        int delay;
+
         g_return_val_if_fail (GDM_IS_DISPLAY (display), FALSE);
 
-        GDM_DISPLAY_GET_CLASS (display)->get_timed_login_details (display, enabled, username, delay);
+        GDM_DISPLAY_GET_CLASS (display)->get_timed_login_details (display, &enabled, &username, &delay);
 
-        g_debug ("GdmSlave: Got timed login details for display %s: %d '%s' %d",
+        g_debug ("GdmDisplay: Got timed login details for display %s: %d '%s' %d",
                  display->priv->x11_display_name,
-                 *enabled,
-                 *username ? *username : "(null)",
-                 *delay);
+                 enabled,
+                 username,
+                 delay);
+
+        if (out_enabled) {
+                *out_enabled = enabled;
+        }
+
+        if (out_username) {
+                *out_username = username;
+        } else {
+                g_free (username);
+        }
+
+        if (out_delay) {
+                *out_delay = delay;
+        }
 
         return TRUE;
 }
@@ -585,9 +610,9 @@ queue_finish (GdmDisplay *display)
 }
 
 static void
-slave_exited (GdmSlaveProxy       *proxy,
-              int                  code,
-              GdmDisplay          *display)
+slave_exited (GdmSlaveJob       *job,
+              int                code,
+              GdmDisplay        *display)
 {
         g_debug ("GdmDisplay: Slave exited: %d", code);
 
@@ -595,9 +620,9 @@ slave_exited (GdmSlaveProxy       *proxy,
 }
 
 static void
-slave_died (GdmSlaveProxy       *proxy,
-            int                  signum,
-            GdmDisplay          *display)
+slave_died (GdmSlaveJob       *job,
+            int                signum,
+            GdmDisplay        *display)
 {
         g_debug ("GdmDisplay: Slave died: %d", signum);
 
@@ -626,7 +651,7 @@ gdm_display_real_prepare (GdmDisplay *display)
 
         g_debug ("GdmDisplay: prepare display");
 
-        g_assert (display->priv->slave_proxy == NULL);
+        g_assert (display->priv->slave_job == NULL);
 
         if (!gdm_display_create_authority (display)) {
                 g_warning ("Unable to set up access control for display %d",
@@ -636,12 +661,12 @@ gdm_display_real_prepare (GdmDisplay *display)
 
         _gdm_display_set_status (display, GDM_DISPLAY_PREPARED);
 
-        display->priv->slave_proxy = gdm_slave_proxy_new ();
-        g_signal_connect (display->priv->slave_proxy,
+        display->priv->slave_job = gdm_slave_job_new ();
+        g_signal_connect (display->priv->slave_job,
                           "exited",
                           G_CALLBACK (slave_exited),
                           display);
-        g_signal_connect (display->priv->slave_proxy,
+        g_signal_connect (display->priv->slave_job,
                           "died",
                           G_CALLBACK (slave_died),
                           display);
@@ -649,13 +674,13 @@ gdm_display_real_prepare (GdmDisplay *display)
         log_file = g_strdup_printf ("%s-slave.log", display->priv->x11_display_name);
         log_path = g_build_filename (LOGDIR, log_file, NULL);
         g_free (log_file);
-        gdm_slave_proxy_set_log_path (display->priv->slave_proxy, log_path);
+        gdm_slave_job_set_log_path (display->priv->slave_job, log_path);
         g_free (log_path);
 
         command = g_strdup_printf ("%s --display-id %s",
                                    display->priv->slave_command,
                                    display->priv->id);
-        gdm_slave_proxy_set_command (display->priv->slave_proxy, command);
+        gdm_slave_job_set_command (display->priv->slave_job, command);
         g_free (command);
 
         return TRUE;
@@ -677,7 +702,6 @@ gdm_display_prepare (GdmDisplay *display)
         return ret;
 }
 
-
 static gboolean
 gdm_display_real_manage (GdmDisplay *display)
 {
@@ -695,11 +719,11 @@ gdm_display_real_manage (GdmDisplay *display)
                 }
         }
 
-        g_assert (display->priv->slave_proxy != NULL);
+        g_assert (display->priv->slave_job != NULL);
 
         g_timer_start (display->priv->slave_timer);
 
-        gdm_slave_proxy_start (display->priv->slave_proxy);
+        gdm_slave_job_start (display->priv->slave_job);
 
         return TRUE;
 }
@@ -759,11 +783,11 @@ gdm_display_real_unmanage (GdmDisplay *display)
 
         g_timer_stop (display->priv->slave_timer);
 
-        if (display->priv->slave_proxy != NULL) {
-                gdm_slave_proxy_stop (display->priv->slave_proxy);
+        if (display->priv->slave_job != NULL) {
+                gdm_slave_job_stop (display->priv->slave_job);
 
-                g_object_unref (display->priv->slave_proxy);
-                display->priv->slave_proxy = NULL;
+                g_object_unref (display->priv->slave_job);
+                display->priv->slave_job = NULL;
         }
 
         if (display->priv->user_access_file != NULL) {
@@ -1333,7 +1357,7 @@ gdm_display_open_session_sync (GdmDisplay    *display,
         char *address;
         int ret;
 
-        if (display->priv->slave_bus_proxy == NULL) {
+        if (display->priv->slave_proxy == NULL) {
                 g_set_error (error,
                              G_DBUS_ERROR,
                              G_DBUS_ERROR_ACCESS_DENIED,
@@ -1342,7 +1366,7 @@ gdm_display_open_session_sync (GdmDisplay    *display,
         }
 
         address = NULL;
-        ret = gdm_dbus_slave_call_open_session_sync (display->priv->slave_bus_proxy,
+        ret = gdm_dbus_slave_call_open_session_sync (display->priv->slave_proxy,
                                                      (int) pid_of_caller,
                                                      (int) uid_of_caller,
                                                      &address,
@@ -1367,7 +1391,7 @@ gdm_display_open_reauthentication_channel_sync (GdmDisplay    *display,
         char *address;
         int ret;
 
-        if (display->priv->slave_bus_proxy == NULL) {
+        if (display->priv->slave_proxy == NULL) {
                 g_set_error (error,
                              G_DBUS_ERROR,
                              G_DBUS_ERROR_ACCESS_DENIED,
@@ -1376,7 +1400,7 @@ gdm_display_open_reauthentication_channel_sync (GdmDisplay    *display,
         }
 
         address = NULL;
-        ret = gdm_dbus_slave_call_open_reauthentication_channel_sync (display->priv->slave_bus_proxy,
+        ret = gdm_dbus_slave_call_open_reauthentication_channel_sync (display->priv->slave_proxy,
                                                                       username,
                                                                       pid_of_caller,
                                                                       uid_of_caller,
@@ -1439,11 +1463,11 @@ gdm_display_dispose (GObject *object)
                 display->priv->finish_idle_id = 0;
         }
 
-        if (display->priv->slave_proxy != NULL) {
-                gdm_slave_proxy_stop (display->priv->slave_proxy);
+        if (display->priv->slave_job != NULL) {
+                gdm_slave_job_stop (display->priv->slave_job);
 
-                g_object_unref (display->priv->slave_proxy);
-                display->priv->slave_proxy = NULL;
+                g_object_unref (display->priv->slave_job);
+                display->priv->slave_job = NULL;
         }
 
         if (display->priv->user_access_file != NULL) {
@@ -1631,12 +1655,6 @@ gdm_display_finalize (GObject *object)
         G_OBJECT_CLASS (gdm_display_parent_class)->finalize (object);
 }
 
-GDBusConnection *
-gdm_display_get_bus_connection (GdmDisplay *display)
-{
-        return display->priv->connection;
-}
-
 GDBusObjectSkeleton *
 gdm_display_get_object_skeleton (GdmDisplay *display)
 {
