@@ -260,24 +260,18 @@ lookup_by_session_id (const char *id,
                       gpointer    user_data)
 {
         const char *looking_for = user_data;
-        char *current;
-        gboolean res;
+        const char *current;
 
         current = gdm_display_get_session_id (display);
-
-        res = g_strcmp0 (current, looking_for) == 0;
-
-        g_free (current);
-
-        return res;
+        return g_strcmp0 (current, looking_for) == 0;
 }
 
 static GdmDisplay *
 get_display_and_details_for_bus_sender (GdmManager       *self,
-                            GDBusConnection  *connection,
-                            const char       *sender,
-                            GPid             *out_pid,
-                            uid_t            *out_uid)
+                                        GDBusConnection  *connection,
+                                        const char       *sender,
+                                        GPid             *out_pid,
+                                        uid_t            *out_uid)
 {
         GdmDisplay *display = NULL;
         char       *session_id = NULL;
@@ -332,11 +326,11 @@ out:
         g_free (session_id);
 
         if (display != NULL) {
-            if (out_pid != NULL)
-                *out_pid = pid;
+                if (out_pid != NULL)
+                        *out_pid = pid;
 
-            if (out_uid != NULL)
-                *out_uid = session_uid;
+                if (out_uid != NULL)
+                        *out_uid = session_uid;
         }
         return display;
 }
