@@ -43,6 +43,14 @@
 #include <linux/vt.h>
 #endif
 
+#ifdef WITH_SYSTEMD
+#include <systemd/sd-daemon.h>
+#endif
+
+#ifdef ENABLE_SYSTEMD_JOURNAL
+#include <systemd/sd-journal.h>
+#endif
+
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
 #include <gio/gio.h>
@@ -85,13 +93,7 @@ struct GdmServerPrivate
         char    *display_seat_id;
         char    *auth_file;
 
-        gboolean is_parented;
-        char    *parent_display_name;
-        char    *parent_auth_file;
-        char    *chosen_hostname;
-
         guint    child_watch_id;
-        guint    sigusr1_id;
 
         gboolean is_initial;
 };
@@ -102,15 +104,7 @@ enum {
         PROP_DISPLAY_SEAT_ID,
         PROP_DISPLAY_DEVICE,
         PROP_AUTH_FILE,
-        PROP_IS_PARENTED,
-        PROP_PARENT_DISPLAY_NAME,
-        PROP_PARENT_AUTH_FILE,
-        PROP_CHOSEN_HOSTNAME,
-        PROP_COMMAND,
-        PROP_PRIORITY,
         PROP_USER_NAME,
-        PROP_SESSION_ARGS,
-        PROP_LOG_DIR,
         PROP_DISABLE_TCP,
         PROP_IS_INITIAL,
 };
@@ -185,16 +179,77 @@ gdm_server_get_display_device (GdmServer *server)
         return g_strdup (server->priv->display_device);
 }
 
-static gboolean
-on_sigusr1 (gpointer user_data)
-
+static void
+gdm_server_ready (GdmServer *server)
 {
-        GdmServer *server = user_data;
-
         g_debug ("GdmServer: Got USR1 from X server - emitting READY");
-
         g_signal_emit (server, signals[READY], 0);
-        return FALSE;
+}
+
+static GSList *active_servers;
+static gboolean sigusr1_thread_running;
+static GCond sigusr1_thread_cond;
+static GMutex sigusr1_thread_mutex;
+
+static gboolean
+got_sigusr1 (gpointer user_data)
+{
+        GPid pid = GPOINTER_TO_UINT (user_data);
+        GSList *l;
+
+        g_debug ("GdmServer: got SIGUSR1 from PID %d", pid);
+
+        for (l = active_servers; l; l = l->next) {
+                GdmServer *server = l->data;
+
+                if (server->priv->pid == pid)
+                        gdm_server_ready (server);
+        }
+
+        return G_SOURCE_REMOVE;
+}
+
+static gpointer
+sigusr1_thread_main (gpointer user_data)
+{
+        sigset_t sigusr1_mask;
+
+        /* Handle only SIGUSR1 */
+        sigemptyset (&sigusr1_mask);
+        sigaddset (&sigusr1_mask, SIGUSR1);
+        sigprocmask (SIG_SETMASK, &sigusr1_mask, NULL);
+
+        g_mutex_lock (&sigusr1_thread_mutex);
+        sigusr1_thread_running = TRUE;
+        g_cond_signal (&sigusr1_thread_cond);
+        g_mutex_unlock (&sigusr1_thread_mutex);
+
+        /* Spin waiting for a SIGUSR1 */
+        while (TRUE) {
+                siginfo_t info;
+
+                if (sigwaitinfo (&sigusr1_mask, &info) == -1)
+                        continue;
+
+                g_idle_add (got_sigusr1, GUINT_TO_POINTER (info.si_pid));
+        }
+
+        return NULL;
+}
+
+static void
+gdm_server_launch_sigusr1_thread_if_needed (void)
+{
+        static GThread *sigusr1_thread;
+
+        if (sigusr1_thread == NULL) {
+                sigusr1_thread = g_thread_new ("gdm SIGUSR1 catcher", sigusr1_thread_main, NULL);
+
+                g_mutex_lock (&sigusr1_thread_mutex);
+                while (!sigusr1_thread_running)
+                        g_cond_wait (&sigusr1_thread_cond, &sigusr1_thread_mutex);
+                g_mutex_unlock (&sigusr1_thread_mutex);
+        }
 }
 
 /* We keep a connection (parent_dsp) open with the parent X server
@@ -243,6 +298,7 @@ gdm_server_init_command (GdmServer *server)
 {
         gboolean debug = FALSE;
         const char *debug_options;
+        const char *verbosity = "";
 
         if (server->priv->command != NULL) {
                 return;
@@ -275,6 +331,15 @@ gdm_server_init_command (GdmServer *server)
                 goto fallback;
         }
 
+#ifdef ENABLE_SYSTEMD_JOURNAL
+        /* For systemd, we don't have a log file but instead log to stdout,
+           so set it to the xserver's built-in default verbosity */
+        if (debug)
+            verbosity = "7 -logfile /dev/null";
+        else
+            verbosity = "3 -logfile /dev/null";
+#endif
+
         if (g_access (SYSTEMD_X_SERVER, X_OK) < 0) {
                 goto fallback;
         }
@@ -284,13 +349,13 @@ gdm_server_init_command (GdmServer *server)
                 goto fallback;
         }
 
-        server->priv->command = g_strdup_printf (SYSTEMD_X_SERVER " -background none -verbose%s", debug_options);
+        server->priv->command = g_strdup_printf (SYSTEMD_X_SERVER " -background none -verbose %s%s", verbosity, debug_options);
         return;
 
 fallback:
 #endif
 
-        server->priv->command = g_strdup_printf (X_SERVER " -background none -verbose%s", debug_options);
+        server->priv->command = g_strdup_printf (X_SERVER " -background none -verbose %s%s", verbosity, debug_options);
 }
 
 static gboolean
@@ -347,14 +412,6 @@ gdm_server_resolve_command_line (GdmServer  *server,
         }
 #endif
 
-        if (server->priv->chosen_hostname) {
-                /* run just one session */
-                argv[len++] = g_strdup ("-terminate");
-                argv[len++] = g_strdup ("-query");
-                argv[len++] = g_strdup (server->priv->chosen_hostname);
-                query_in_arglist = TRUE;
-        }
-
         if (server->priv->disable_tcp && ! query_in_arglist) {
                 argv[len++] = g_strdup ("-nolisten");
                 argv[len++] = g_strdup ("tcp");
@@ -451,12 +508,44 @@ change_user (GdmServer *server)
         }
 }
 
+static gboolean
+gdm_server_setup_journal_fds (GdmServer *server)
+{
+#ifdef ENABLE_SYSTEMD_JOURNAL
+    if (sd_booted () > 0) {
+        int out, err;
+        const char *prefix = "gdm-Xorg-";
+        char *identifier;
+        gsize size;
+
+        size = strlen (prefix) + strlen (server->priv->display_name) + 1;
+        identifier = g_alloca (size);
+        strcpy (identifier, prefix);
+        strcat (identifier, server->priv->display_name);
+        identifier[size - 1] = '\0';
+
+        out = sd_journal_stream_fd (identifier, LOG_INFO, FALSE);
+        if (out < 0)
+            return FALSE;
+
+        err = sd_journal_stream_fd (identifier, LOG_WARNING, FALSE);
+        if (err < 0) {
+            close (out);
+            return FALSE;
+        }
+
+        VE_IGNORE_EINTR (dup2 (out, 1));
+        VE_IGNORE_EINTR (dup2 (err, 2));
+        return TRUE;
+    }
+#endif
+    return FALSE;
+}
+
 static void
-server_child_setup (GdmServer *server)
+gdm_server_setup_logfile (GdmServer *server)
 {
         int              logfd;
-        struct sigaction ign_signal;
-        sigset_t         mask;
         char            *log_file;
         char            *log_path;
 
@@ -467,7 +556,6 @@ server_child_setup (GdmServer *server)
         /* Rotate the X server logs */
         rotate_logs (log_path, MAX_LOGS);
 
-        /* Log all output from spawned programs to a file */
         g_debug ("GdmServer: Opening logfile for server %s", log_path);
 
         VE_IGNORE_EINTR (g_unlink (log_path));
@@ -484,6 +572,16 @@ server_child_setup (GdmServer *server)
                            "gdm_server_spawn",
                            server->priv->display_name);
         }
+}
+
+static void
+server_child_setup (GdmServer *server)
+{
+        struct sigaction ign_signal;
+        sigset_t         mask;
+
+        if (!gdm_server_setup_journal_fds(server))
+            gdm_server_setup_logfile(server);
 
         /* The X server expects USR1/TTIN/TTOU to be SIG_IGN */
         ign_signal.sa_handler = SIG_IGN;
@@ -564,17 +662,7 @@ get_server_environment (GdmServer *server)
         }
 
         /* modify environment here */
-        if (server->priv->is_parented) {
-                if (server->priv->parent_auth_file != NULL) {
-                        g_hash_table_insert (hash, g_strdup ("XAUTHORITY"), g_strdup (server->priv->parent_auth_file));
-                }
-
-                if (server->priv->parent_display_name != NULL) {
-                        g_hash_table_insert (hash, g_strdup ("DISPLAY"), g_strdup (server->priv->parent_display_name));
-                }
-        } else {
-                g_hash_table_insert (hash, g_strdup ("DISPLAY="), g_strdup (server->priv->display_name));
-        }
+        g_hash_table_insert (hash, g_strdup ("DISPLAY"), g_strdup (server->priv->display_name));
 
         if (server->priv->user_name != NULL) {
                 struct passwd *pwent;
@@ -648,6 +736,8 @@ server_child_watch (GPid       pid,
                 g_signal_emit (server, signals [DIED], 0, num);
         }
 
+        active_servers = g_slist_remove (active_servers, server);
+
         g_spawn_close_pid (server->priv->pid);
         server->priv->pid = -1;
 
@@ -691,6 +781,10 @@ gdm_server_spawn (GdmServer    *server,
         g_debug ("GdmServer: Starting X server process: %s", freeme);
         g_free (freeme);
 
+        active_servers = g_slist_append (active_servers, server);
+
+        gdm_server_launch_sigusr1_thread_if_needed ();
+
         if (!g_spawn_async_with_pipes (NULL,
                                        argv,
                                        (char **)env->pdata,
@@ -1035,16 +1129,11 @@ gdm_server_class_init (GdmServerClass *klass)
 static void
 gdm_server_init (GdmServer *server)
 {
-
         server->priv = GDM_SERVER_GET_PRIVATE (server);
 
         server->priv->pid = -1;
 
         server->priv->log_dir = g_strdup (LOGDIR);
-
-        server->priv->sigusr1_id = g_unix_signal_add (SIGUSR1,
-                                                      on_sigusr1,
-                                                      server);
 }
 
 static void
@@ -1059,9 +1148,6 @@ gdm_server_finalize (GObject *object)
 
         g_return_if_fail (server->priv != NULL);
 
-        if (server->priv->sigusr1_id > 0)
-                g_source_remove (server->priv->sigusr1_id);
-
         gdm_server_stop (server);
 
         g_free (server->priv->command);
@@ -1072,9 +1158,6 @@ gdm_server_finalize (GObject *object)
         g_free (server->priv->display_seat_id);
         g_free (server->priv->display_device);
         g_free (server->priv->auth_file);
-        g_free (server->priv->parent_display_name);
-        g_free (server->priv->parent_auth_file);
-        g_free (server->priv->chosen_hostname);
 
         G_OBJECT_CLASS (gdm_server_parent_class)->finalize (object);
 }
