@@ -31,7 +31,6 @@
 #include <errno.h>
 #include <pwd.h>
 #include <grp.h>
-#include <signal.h>
 
 #include <glib.h>
 #include <glib/gstdio.h>
@@ -73,8 +72,6 @@
 
 #define GDM_SLAVE_PATH "/org/gnome/DisplayManager/Slave"
 
-#define MAX_CONNECT_ATTEMPTS 10
-
 struct GdmSlavePrivate
 {
         GPid             pid;
@@ -91,11 +88,8 @@ struct GdmSlavePrivate
         int              display_number;
         char            *display_hostname;
         gboolean         display_is_local;
-        gboolean         display_is_parented;
         char            *display_seat_id;
         char            *display_x11_authority_file;
-        char            *parent_display_name;
-        char            *parent_display_x11_authority_file;
         char            *windowpath;
         GBytes          *display_x11_cookie;
         gboolean         display_is_initial;
@@ -200,7 +194,6 @@ get_script_environment (GdmSlave   *slave,
         GPtrArray     *env;
         GHashTable    *hash;
         struct passwd *pwent;
-        char          *temp;
 
         env = g_ptr_array_new ();
 
@@ -234,14 +227,6 @@ get_script_environment (GdmSlave   *slave,
                 }
         }
 
-#if 0
-        if (display_is_parented) {
-                g_hash_table_insert (hash, g_strdup ("GDM_PARENT_DISPLAY"), g_strdup (parent_display_name));
-
-                /*g_hash_table_insert (hash, "GDM_PARENT_XAUTHORITY"), slave->priv->parent_temp_auth_file));*/
-        }
-#endif
-
         if (! slave->priv->display_is_local) {
                 g_hash_table_insert (hash, g_strdup ("REMOTE_HOST"), g_strdup (slave->priv->display_hostname));
         }
@@ -492,89 +477,10 @@ gdm_slave_setup_xhost_auth (XHostAddress *host_entries, XServerInterpretedAddres
         host_entries[2].length    = sizeof (XServerInterpretedAddress);
 }
 
-static void
-gdm_slave_set_windowpath (GdmSlave *slave)
-{
-        /* setting WINDOWPATH for clients */
-        Atom prop;
-        Atom actualtype;
-        int actualformat;
-        unsigned long nitems;
-        unsigned long bytes_after;
-        unsigned char *buf;
-        const char *windowpath;
-        char *newwindowpath;
-        unsigned long num;
-        char nums[10];
-        int numn;
-
-        prop = XInternAtom (slave->priv->server_display, "XFree86_VT", False);
-        if (prop == None) {
-                g_debug ("no XFree86_VT atom\n");
-                return;
-        }
-        if (XGetWindowProperty (slave->priv->server_display,
-                DefaultRootWindow (slave->priv->server_display), prop, 0, 1,
-                False, AnyPropertyType, &actualtype, &actualformat,
-                &nitems, &bytes_after, &buf)) {
-                g_debug ("no XFree86_VT property\n");
-                return;
-        }
-
-        if (nitems != 1) {
-                g_debug ("%lu items in XFree86_VT property!\n", nitems);
-                XFree (buf);
-                return;
-        }
-
-        switch (actualtype) {
-        case XA_CARDINAL:
-        case XA_INTEGER:
-        case XA_WINDOW:
-                switch (actualformat) {
-                case  8:
-                        num = (*(uint8_t  *)(void *)buf);
-                        break;
-                case 16:
-                        num = (*(uint16_t *)(void *)buf);
-                        break;
-                case 32:
-                        num = (*(long *)(void *)buf);
-                        break;
-                default:
-                        g_debug ("format %d in XFree86_VT property!\n", actualformat);
-                        XFree (buf);
-                        return;
-                }
-                break;
-        default:
-                g_debug ("type %lx in XFree86_VT property!\n", actualtype);
-                XFree (buf);
-                return;
-        }
-        XFree (buf);
-
-        windowpath = getenv ("WINDOWPATH");
-        numn = snprintf (nums, sizeof (nums), "%lu", num);
-        if (!windowpath) {
-                newwindowpath = malloc (numn + 1);
-                sprintf (newwindowpath, "%s", nums);
-        } else {
-                newwindowpath = malloc (strlen (windowpath) + 1 + numn + 1);
-                sprintf (newwindowpath, "%s:%s", windowpath, nums);
-        }
-
-        slave->priv->windowpath = newwindowpath;
-
-        g_setenv ("WINDOWPATH", newwindowpath, TRUE);
-}
-
 gboolean
 gdm_slave_connect_to_x11_display (GdmSlave *slave)
 {
         gboolean ret;
-        sigset_t mask;
-        sigset_t omask;
 
         ret = FALSE;
 
@@ -583,13 +489,6 @@ gdm_slave_connect_to_x11_display (GdmSlave *slave)
 
         g_debug ("GdmSlave: Server is ready - opening display %s", slave->priv->display_name);
 
-        g_setenv ("DISPLAY", slave->priv->display_name, TRUE);
-        g_setenv ("XAUTHORITY", slave->priv->display_x11_authority_file, TRUE);
-
-        sigemptyset (&mask);
-        sigaddset (&mask, SIGCHLD);
-        sigprocmask (SIG_BLOCK, &mask, &omask);
-
         /* Give slave access to the display independent of current hostname */
         if (slave->priv->display_x11_cookie != NULL) {
                 XSetAuthorization ("MIT-MAGIC-COOKIE-1",
@@ -601,9 +500,6 @@ gdm_slave_connect_to_x11_display (GdmSlave *slave)
 
         slave->priv->server_display = XOpenDisplay (slave->priv->display_name);
 
-        sigprocmask (SIG_SETMASK, &omask, NULL);
-
-
         if (slave->priv->server_display == NULL) {
                 g_warning ("Unable to connect to display %s", slave->priv->display_name);
                 ret = FALSE;
@@ -630,8 +526,6 @@ gdm_slave_connect_to_x11_display (GdmSlave *slave)
                 if (gdm_error_trap_pop ()) {
                         g_warning ("Failed to give slave programs access to the display. Trying to proceed.");
                 }
-
-                gdm_slave_set_windowpath (slave);
         } else {
                 g_debug ("GdmSlave: Connected to display %s", slave->priv->display_name);
                 ret = TRUE;
@@ -666,7 +560,7 @@ static gboolean
 gdm_slave_real_start (GdmSlave *slave)
 {
         gboolean    res;
-        char       *id;
+        char       *id = NULL;
         GError     *error;
         GVariant   *x11_cookie;
         const char *x11_cookie_bytes;
@@ -676,6 +570,11 @@ gdm_slave_real_start (GdmSlave *slave)
 
         g_assert (slave->priv->display_proxy == NULL);
 
+        if (! g_variant_is_object_path (slave->priv->display_id)) {
+                g_warning ("Display ID isn't valid");
+                return FALSE;
+        }
+
         g_debug ("GdmSlave: Creating proxy for %s", slave->priv->display_id);
         error = NULL;
         slave->priv->display_proxy = GDM_DBUS_DISPLAY (gdm_dbus_display_proxy_new_sync (slave->priv->connection,
@@ -696,7 +595,7 @@ gdm_slave_real_start (GdmSlave *slave)
                                                  &id,
                                                  NULL,
                                                  &error);
-        if (! res) {
+        if (! res || !id) {
                 g_warning ("Failed to get display ID %s: %s", slave->priv->display_id, error->message);
                 g_error_free (error);
                 return FALSE;
@@ -705,8 +604,8 @@ gdm_slave_real_start (GdmSlave *slave)
         g_debug ("GdmSlave: Got display ID: %s", id);
 
         if (strcmp (id, slave->priv->display_id) != 0) {
-                g_critical ("Display ID doesn't match");
-                exit (1);
+                g_warning ("Display ID doesn't match");
+                return FALSE;
         }
 
         gdm_slave_set_slave_bus_name (slave);
@@ -1706,59 +1605,6 @@ _gdm_slave_set_display_id (GdmSlave   *slave,
         slave->priv->display_id = g_strdup (id);
 }
 
-static void
-_gdm_slave_set_display_name (GdmSlave   *slave,
-                             const char *name)
-{
-        g_free (slave->priv->display_name);
-        slave->priv->display_name = g_strdup (name);
-}
-
-static void
-_gdm_slave_set_display_number (GdmSlave   *slave,
-                               int         number)
-{
-        slave->priv->display_number = number;
-}
-
-static void
-_gdm_slave_set_display_hostname (GdmSlave   *slave,
-                                 const char *name)
-{
-        g_free (slave->priv->display_hostname);
-        slave->priv->display_hostname = g_strdup (name);
-}
-
-static void
-_gdm_slave_set_display_x11_authority_file (GdmSlave   *slave,
-                                           const char *name)
-{
-        g_free (slave->priv->display_x11_authority_file);
-        slave->priv->display_x11_authority_file = g_strdup (name);
-}
-
-static void
-_gdm_slave_set_display_seat_id (GdmSlave   *slave,
-                                const char *id)
-{
-        g_free (slave->priv->display_seat_id);
-        slave->priv->display_seat_id = g_strdup (id);
-}
-
-static void
-_gdm_slave_set_display_is_local (GdmSlave   *slave,
-                                 gboolean    is)
-{
-        slave->priv->display_is_local = is;
-}
-
-static void
-_gdm_slave_set_display_is_initial (GdmSlave   *slave,
-                                   gboolean    is)
-{
-        slave->priv->display_is_initial = is;
-}
-
 static void
 gdm_slave_set_property (GObject      *object,
                         guint         prop_id,
@@ -1776,27 +1622,6 @@ gdm_slave_set_property (GObject      *object,
         case PROP_DISPLAY_ID:
                 _gdm_slave_set_display_id (self, g_value_get_string (value));
                 break;
-        case PROP_DISPLAY_NAME:
-                _gdm_slave_set_display_name (self, g_value_get_string (value));
-                break;
-        case PROP_DISPLAY_NUMBER:
-                _gdm_slave_set_display_number (self, g_value_get_int (value));
-                break;
-        case PROP_DISPLAY_HOSTNAME:
-                _gdm_slave_set_display_hostname (self, g_value_get_string (value));
-                break;
-        case PROP_DISPLAY_SEAT_ID:
-                _gdm_slave_set_display_seat_id (self, g_value_get_string (value));
-                break;
-        case PROP_DISPLAY_X11_AUTHORITY_FILE:
-                _gdm_slave_set_display_x11_authority_file (self, g_value_get_string (value));
-                break;
-        case PROP_DISPLAY_IS_LOCAL:
-                _gdm_slave_set_display_is_local (self, g_value_get_boolean (value));
-                break;
-        case PROP_DISPLAY_IS_INITIAL:
-                _gdm_slave_set_display_is_initial (self, g_value_get_boolean (value));
-                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
@@ -1856,7 +1681,7 @@ handle_open_session (GdmDBusSlave          *skeleton,
 {
         GError        *error;
         GdmSlaveClass *slave_class;
-        char          *address;
+        const char    *address;
 
         slave_class = GDM_SLAVE_GET_CLASS (slave);
         if (slave_class->open_session == NULL) {
@@ -1880,7 +1705,6 @@ handle_open_session (GdmDBusSlave          *skeleton,
 
         gdm_dbus_slave_complete_open_session (skeleton, invocation, address);
 
-        g_free (address);
         return TRUE;
 }
 
@@ -2035,7 +1859,7 @@ gdm_slave_class_init (GdmSlaveClass *klass)
                                                               "display name",
                                                               "display name",
                                                               NULL,
-                                                              G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+                                                              G_PARAM_READABLE));
         g_object_class_install_property (object_class,
                                          PROP_DISPLAY_NUMBER,
                                          g_param_spec_int ("display-number",
@@ -2044,42 +1868,42 @@ gdm_slave_class_init (GdmSlaveClass *klass)
                                                            -1,
                                                            G_MAXINT,
                                                            -1,
-                                                           G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+                                                           G_PARAM_READABLE));
         g_object_class_install_property (object_class,
                                          PROP_DISPLAY_HOSTNAME,
                                          g_param_spec_string ("display-hostname",
                                                               "display hostname",
                                                               "display hostname",
                                                               NULL,
-                                                              G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+                                                              G_PARAM_READABLE));
         g_object_class_install_property (object_class,
                                          PROP_DISPLAY_SEAT_ID,
                                          g_param_spec_string ("display-seat-id",
                                                               "",
                                                               "",
                                                               NULL,
-                                                              G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+                                                              G_PARAM_READABLE));
         g_object_class_install_property (object_class,
                                          PROP_DISPLAY_X11_AUTHORITY_FILE,
                                          g_param_spec_string ("display-x11-authority-file",
                                                               "",
                                                               "",
                                                               NULL,
-                                                              G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+                                                              G_PARAM_READABLE));
         g_object_class_install_property (object_class,
                                          PROP_DISPLAY_IS_LOCAL,
                                          g_param_spec_boolean ("display-is-local",
                                                                "display is local",
                                                                "display is local",
                                                                TRUE,
-                                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+                                                               G_PARAM_READABLE));
         g_object_class_install_property (object_class,
                                          PROP_DISPLAY_IS_INITIAL,
                                          g_param_spec_boolean ("display-is-initial",
                                                                NULL,
                                                                NULL,
                                                                FALSE,
-                                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+                                                               G_PARAM_READABLE));
 
         signals [STOPPED] =
                 g_signal_new ("stopped",
@@ -2121,9 +1945,6 @@ gdm_slave_finalize (GObject *object)
         g_free (slave->priv->display_hostname);
         g_free (slave->priv->display_seat_id);
         g_free (slave->priv->display_x11_authority_file);
-        g_free (slave->priv->parent_display_name);
-        g_free (slave->priv->parent_display_x11_authority_file);
-        g_free (slave->priv->windowpath);
         g_bytes_unref (slave->priv->display_x11_cookie);
 
         G_OBJECT_CLASS (gdm_slave_parent_class)->finalize (object);
