@@ -184,7 +184,6 @@ create_chooser_session (const char *display_name,
 static void
 run_chooser (GdmXdmcpChooserSlave *slave)
 {
-        char          *display_id;
         char          *display_name;
         char          *display_device;
         char          *display_hostname;
@@ -194,14 +193,12 @@ run_chooser (GdmXdmcpChooserSlave *slave)
 
         g_debug ("GdmXdmcpChooserSlave: Running chooser");
 
-        display_id = NULL;
         display_name = NULL;
         auth_file = NULL;
         display_device = NULL;
         display_hostname = NULL;
 
         g_object_get (slave,
-                      "display-id", &display_id,
                       "display-name", &display_name,
                       "display-hostname", &display_hostname,
                       "display-x11-authority-file", &auth_file,
@@ -271,7 +268,6 @@ run_chooser (GdmXdmcpChooserSlave *slave)
                           "client-connected",
                           G_CALLBACK (on_chooser_connected),
                           slave);
-        g_free (display_id);
         g_free (display_name);
         g_free (display_device);
         g_free (display_hostname);
@@ -350,11 +346,11 @@ gdm_xdmcp_chooser_slave_stop (GdmSlave *slave)
 }
 
 static gboolean
-gdm_xdmcp_chooser_slave_open_session (GdmSlave  *slave,
-                                      GPid       pid_of_caller,
-                                      uid_t      uid_of_caller,
-                                      char     **address,
-                                      GError   **error)
+gdm_xdmcp_chooser_slave_open_session (GdmSlave    *slave,
+                                      GPid         pid_of_caller,
+                                      uid_t        uid_of_caller,
+                                      const char **address,
+                                      GError     **error)
 {
         GdmXdmcpChooserSlave *self = GDM_XDMCP_CHOOSER_SLAVE (slave);
         GdmSession           *session;
